# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import sys
import unittest

import gphoto2 as gp

path = os.path.dirname(os.path.dirname(__file__))
if path not in sys.path:
    sys.path.insert(0, path)
from tests.vcamera import has_vcam, use_vcam


@unittest.skipUnless(has_vcam, 'no virtual camera')
class TestAbilitiesList(unittest.TestCase):
    def setUp(self):
        use_vcam(True)
        # set locale dir (should be already set by package __init__)
        locale_dir = os.environ['IOLIBS'].replace('vusb', 'locale')
        self.assertEqual(gp.gp_init_localedir(locale_dir), gp.GP_OK)
        self.assertEqual(gp.gp_message_codeset('utf-8'), 'utf-8')

    def test_oo_style(self):
        abilities_list = gp.CameraAbilitiesList()
        self.assertIsInstance(abilities_list, gp.CameraAbilitiesList)
        self.assertEqual(len(abilities_list), 0)
        abilities_list.load()
        self.assertGreater(len(abilities_list), 0)
        abilities = abilities_list[0]
        self.assertIsInstance(abilities, gp.CameraAbilities)
        self.assertIsInstance(abilities.model, str)
        port_info_list = gp.PortInfoList()
        port_info_list.load()
        camera_list = abilities_list.detect(port_info_list)
        self.assertEqual(len(camera_list), 1)
        name, value = camera_list[0]
        self.assertEqual(name, 'Nikon DSC D750')
        self.assertIsInstance(abilities_list.lookup_model(name), int)
        abilities_list.reset()
        self.assertEqual(len(abilities_list), 0)
        abilities_list.load_dir(os.environ['CAMLIBS'])
        self.assertGreater(len(abilities_list), 0)

    def test_c_style(self):
        OK, abilities_list = gp.gp_abilities_list_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(abilities_list, gp.CameraAbilitiesList)
        self.assertEqual(gp.gp_abilities_list_count(abilities_list), 0)
        self.assertEqual(gp.gp_abilities_list_load(abilities_list), gp.GP_OK)
        self.assertGreater(gp.gp_abilities_list_count(abilities_list), 0)
        OK, abilities = gp.gp_abilities_list_get_abilities(abilities_list, 0)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(abilities, gp.CameraAbilities)
        self.assertIsInstance(abilities.model, str)
        OK, port_info_list = gp.gp_port_info_list_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_port_info_list_load(port_info_list), gp.GP_OK)
        OK, camera_list = gp.gp_abilities_list_detect(
            abilities_list, port_info_list)
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_list_count(camera_list), 1)
        OK, name = gp.gp_list_get_name(camera_list, 0)
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(name, 'Nikon DSC D750')
        OK, value = gp.gp_list_get_value(camera_list, 0)
        self.assertEqual(OK, gp.GP_OK)
        idx = gp.gp_abilities_list_lookup_model(abilities_list, name)
        self.assertIsInstance(idx, int)
        self.assertEqual(gp.gp_abilities_list_reset(abilities_list), gp.GP_OK)
        self.assertEqual(gp.gp_abilities_list_count(abilities_list), 0)
        self.assertEqual(
            gp.gp_abilities_list_load_dir(abilities_list,
                                          os.environ['CAMLIBS']), gp.GP_OK)
        self.assertGreater(gp.gp_abilities_list_count(abilities_list), 0)


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import tempfile
import unittest

import gphoto2 as gp


class TestFile(unittest.TestCase):
    def setUp(self):
        self.test_file = os.path.join(
            os.path.dirname(__file__), 'vcamera', 'copyright-free-image.jpg')
        with open(self.test_file, 'rb') as f:
            self.src_data = f.read()
        self.assertEqual(len(self.src_data), os.path.getsize(self.test_file))
        self.assertEqual(self.src_data[:10], b'\xff\xd8\xff\xe0\x00\x10JFIF')

    def test_oo_style(self):
        # create CameraFile from data
        cam_file = gp.CameraFile()
        cam_file.set_data_and_size(self.src_data)
        # get mime type from data
        cam_file.detect_mime_type()
        # check detected mime type
        self.assertEqual(cam_file.get_mime_type(), 'image/jpeg')
        # set mime type anyway
        cam_file.set_mime_type('image/jpeg')
        file_time = int(os.path.getmtime(self.test_file))
        cam_file.set_mtime(file_time)
        file_name = 'cam_file.jpg'
        cam_file.set_name(file_name)
        # read data from CameraFile
        self.assertEqual(cam_file.get_data_and_size(), self.src_data)
        self.assertEqual(cam_file.get_mime_type(), 'image/jpeg')
        self.assertEqual(cam_file.get_mtime(), file_time)
        self.assertEqual(cam_file.get_name(), file_name)
        self.assertEqual(
            cam_file.get_name_by_type(file_name, gp.GP_FILE_TYPE_RAW),
            'raw_' + file_name)
        self.assertEqual(cam_file.detect_mime_type(), None)
        # copy file
        file_copy = gp.CameraFile()
        file_copy.copy(cam_file)
        self.assertEqual(file_copy.get_data_and_size(), self.src_data)
        del file_copy
        # save CameraFile to computer
        with tempfile.TemporaryDirectory() as tmp_dir:
            temp_file = os.path.join(tmp_dir, file_name)
            cam_file.save(temp_file)
            self.assertEqual(os.path.getsize(temp_file), len(self.src_data))
            with open(temp_file, 'rb') as f:
                self.assertEqual(f.read(), self.src_data)
            self.assertEqual(int(os.path.getmtime(temp_file)), file_time)
        # wipe file data
        cam_file.clean()
        self.assertEqual(cam_file.get_data_and_size(), b'')
        self.assertEqual(cam_file.get_name(), '')
        del cam_file
        # open file directly
        direct_file = gp.CameraFile()
        direct_file.open(self.test_file)
        self.assertEqual(direct_file.get_data_and_size(), self.src_data)
        self.assertEqual(direct_file.get_mtime(), file_time)
        self.assertEqual(
            direct_file.get_name(), os.path.basename(self.test_file))
        del direct_file
        # create file from file descriptor
        file_copy = gp.CameraFile(os.open(self.test_file, os.O_RDONLY))
        self.assertEqual(file_copy.get_data_and_size(), self.src_data)
        del file_copy

    def test_c_style(self):
        # create CameraFile from data
        OK, cam_file = gp.gp_file_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(
            gp.gp_file_set_data_and_size(cam_file, self.src_data), gp.GP_OK)
        # get mime type from data
        self.assertEqual(gp.gp_file_detect_mime_type(cam_file), gp.GP_OK)
        # check detected mime type
        self.assertEqual(gp.gp_file_get_mime_type(cam_file),
                         [gp.GP_OK, 'image/jpeg'])
        # set mime type anyway
        self.assertEqual(
            gp.gp_file_set_mime_type(cam_file, 'image/jpeg'), gp.GP_OK)
        file_time = int(os.path.getmtime(self.test_file))
        self.assertEqual(gp.gp_file_set_mtime(cam_file, file_time), gp.GP_OK)
        file_name = 'cam_file.jpg'
        self.assertEqual(gp.gp_file_set_name(cam_file, file_name), gp.GP_OK)
        # read data from CameraFile
        self.assertEqual(gp.gp_file_get_data_and_size(cam_file),
                         [gp.GP_OK, self.src_data])
        self.assertEqual(gp.gp_file_get_mime_type(cam_file),
                         [gp.GP_OK, 'image/jpeg'])
        self.assertEqual(gp.gp_file_get_mtime(cam_file), [gp.GP_OK, file_time])
        self.assertEqual(gp.gp_file_get_name(cam_file), [gp.GP_OK, file_name])
        self.assertEqual(gp.gp_file_get_name_by_type(
            cam_file, file_name, gp.GP_FILE_TYPE_RAW),
                         [gp.GP_OK, 'raw_' + file_name])
        self.assertEqual(gp.gp_file_detect_mime_type(cam_file), gp.GP_OK)
        # copy file
        OK, file_copy = gp.gp_file_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_file_copy(file_copy, cam_file), gp.GP_OK)
        self.assertEqual(gp.gp_file_get_data_and_size(file_copy),
                         [gp.GP_OK, self.src_data])
        del file_copy
        # save CameraFile to computer
        with tempfile.TemporaryDirectory() as tmp_dir:
            temp_file = os.path.join(tmp_dir, file_name)
            self.assertEqual(gp.gp_file_save(cam_file, temp_file), gp.GP_OK)
            self.assertEqual(os.path.getsize(temp_file), len(self.src_data))
            with open(temp_file, 'rb') as f:
                self.assertEqual(f.read(), self.src_data)
            self.assertEqual(int(os.path.getmtime(temp_file)), file_time)
        # wipe file data
        self.assertEqual(gp.gp_file_clean(cam_file), gp.GP_OK)
        self.assertEqual(gp.gp_file_get_data_and_size(cam_file),
                         [gp.GP_OK, b''])
        self.assertEqual(gp.gp_file_get_name(cam_file), [gp.GP_OK, ''])
        del cam_file
        # open file directly
        OK, direct_file = gp.gp_file_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_file_open(direct_file, self.test_file), gp.GP_OK)
        self.assertEqual(gp.gp_file_get_data_and_size(direct_file),
                         [gp.GP_OK, self.src_data])
        self.assertEqual(gp.gp_file_get_mtime(direct_file),
                         [gp.GP_OK, file_time])
        self.assertEqual(gp.gp_file_get_name(direct_file),
                         [gp.GP_OK, os.path.basename(self.test_file)])
        del direct_file
        # create file from file descriptor
        OK, file_copy = gp.gp_file_new_from_fd(
            os.open(self.test_file, os.O_RDONLY))
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_file_get_data_and_size(file_copy),
                         [gp.GP_OK, self.src_data])
        del file_copy


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os

import gphoto2 as gp


def use_vcam(enable):
    if enable:
        os.environ['VCAMERADIR'] = os.path.join(
            os.path.dirname(__file__), 'vcamera')
        # switch to virtual camera from normal drivers
        os.environ['IOLIBS'] = os.environ['IOLIBS'].replace('iolibs', 'vusb')
    else:
        # switch from virtual camera to normal drivers
        os.environ['IOLIBS'] = os.environ['IOLIBS'].replace('vusb', 'iolibs')


def _has_vcam():
    vusb_dir = os.environ['IOLIBS'].replace('iolibs', 'vusb')
    if not os.path.isdir(vusb_dir):
        return False
    gp_version = gp.gp_library_version(gp.GP_VERSION_SHORT)[0]
    gp_version = tuple(int(x) for x in gp_version.split('.'))
    if gp_version > (2, 5, 30):
        return True
    return False

has_vcam = _has_vcam()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import logging
import sys
import unittest

import gphoto2 as gp
from gphoto2.port_log import _gphoto2_logger_cb


class TestPortLog(unittest.TestCase):
    def callback(self, level, domain, string, data):
        self.assertEqual(level, gp.GP_LOG_DEBUG)
        self.assertEqual(domain, 'domain')
        self.assertEqual(string, 'string')
        self.assertEqual(data, 'data')

    def test_oo_style(self):
        callback = gp.Log.add_func(gp.GP_LOG_DEBUG, self.callback, 'data')
        gp.gp_log(gp.GP_LOG_DEBUG, 'domain', 'string')
        del callback
        gp.gp_log(gp.GP_LOG_DEBUG, 'wrong', 'wrong')

    def test_c_style(self):
        OK, callback = gp.gp_log_add_func(
            gp.GP_LOG_DEBUG, self.callback, 'data')
        self.assertGreaterEqual(OK, gp.GP_OK)
        gp.gp_log(gp.GP_LOG_DEBUG, 'domain', 'string')
        del callback
        gp.gp_log(gp.GP_LOG_DEBUG, 'wrong', 'wrong')

    def test_use_python_logging(self):
        self.assertEqual(sys.getrefcount(_gphoto2_logger_cb), 3)
        # test default mapping
        # GP_LOG_DATA maps to DEBUG - 5 which self.assertLogs doesn't handle
        OK, callbacks = gp.use_python_logging()
        self.assertGreaterEqual(OK, gp.GP_OK)
        self.assertEqual(sys.getrefcount(_gphoto2_logger_cb), 4)
        with self.assertLogs('gphoto2', logging.DEBUG):
            gp.gp_log(gp.GP_LOG_DEBUG, 'debug', 'debug_message')
        with self.assertLogs('gphoto2', logging.INFO):
            gp.gp_log(gp.GP_LOG_VERBOSE, 'verbose', 'verbose_message')
        with self.assertLogs('gphoto2', logging.WARNING):
            gp.gp_log(gp.GP_LOG_ERROR, 'error', 'error_message')
        del callbacks
        self.assertEqual(sys.getrefcount(_gphoto2_logger_cb), 3)
        # test custom mapping
        OK, callbacks = gp.use_python_logging(mapping={
            gp.GP_LOG_ERROR   : logging.CRITICAL,
            gp.GP_LOG_VERBOSE : logging.ERROR,
            gp.GP_LOG_DEBUG   : logging.WARNING,
            gp.GP_LOG_DATA    : logging.WARNING})
        self.assertGreaterEqual(OK, gp.GP_OK)
        with self.assertLogs('gphoto2') as cm:
            gp.gp_log(gp.GP_LOG_DATA, 'data', 'data_message')
            gp.gp_log(gp.GP_LOG_DEBUG, 'debug', 'debug_message')
            gp.gp_log(gp.GP_LOG_VERBOSE, 'verbose', 'verbose_message')
            gp.gp_log(gp.GP_LOG_ERROR, 'error', 'error_message')
        self.assertEqual(cm.output, ['WARNING:gphoto2:(data) data_message',
                                     'WARNING:gphoto2:(debug) debug_message',
                                     'ERROR:gphoto2:(verbose) verbose_message',
                                     'CRITICAL:gphoto2:(error) error_message'])


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import sys
import unittest

import gphoto2 as gp

path = os.path.dirname(os.path.dirname(__file__))
if path not in sys.path:
    sys.path.insert(0, path)
from tests.vcamera import use_vcam


class TestPortInfoList(unittest.TestCase):
    def setUp(self):
        use_vcam(False)

    def test_oo_style(self):
        port_info_list = gp.PortInfoList()
        self.assertIsInstance(port_info_list, gp.PortInfoList)
        self.assertEqual(len(port_info_list), 0)
        port_info_list.load()
        self.assertGreater(len(port_info_list), 0)
        port_info = port_info_list.get_info(0)
        name = port_info.get_name()
        self.assertIsInstance(name, str)
        self.assertEqual(port_info_list.lookup_name(name), 0)
        path = port_info.get_path()
        self.assertIsInstance(path, str)
        self.assertEqual(port_info_list.lookup_path(path), 0)
        type_ = port_info.get_type()
        self.assertIsInstance(type_, int)

    def test_c_style(self):
        OK, port_info_list = gp.gp_port_info_list_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(port_info_list, gp.PortInfoList)
        self.assertEqual(gp.gp_port_info_list_count(port_info_list), 0)
        self.assertEqual(gp.gp_port_info_list_load(port_info_list), gp.GP_OK)
        self.assertGreater(gp.gp_port_info_list_count(port_info_list), 0)
        OK, port_info = gp.gp_port_info_list_get_info(port_info_list, 0)
        self.assertEqual(OK, gp.GP_OK)
        OK, name = gp.gp_port_info_get_name(port_info)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(name, str)
        self.assertEqual(
            gp.gp_port_info_list_lookup_name(port_info_list, name), 0)
        OK, path = gp.gp_port_info_get_path(port_info)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(path, str)
        self.assertEqual(
            gp.gp_port_info_list_lookup_path(port_info_list, path), 0)
        OK, type_ = gp.gp_port_info_get_type(port_info)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(type_, int)


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2021  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import sys

# requires GitPython - 'sudo pip install gitpython'
import git


def main(argv=None):
    # get root dir
    root = os.path.dirname(os.path.dirname(os.path.abspath(sys.argv[0])))
    # get python-gphoto2 version
    with open(os.path.join(root, 'README.rst')) as rst:
        version = rst.readline().split()[-1]
    message = 'v' + version + '\n\n'
    with open(os.path.join(root, 'CHANGELOG.txt')) as cl:
        while not cl.readline().startswith('Changes'):
            pass
        while True:
            line = cl.readline().strip()
            if not line:
                break
            message += line + '\n'
    repo = git.Repo()
    tag = repo.create_tag('v' + version, message=message)
    remote = repo.remotes.origin
    remote.push(tags=True)
    return 0

if __name__ == "__main__":
    sys.exit(main())



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2021-23  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import re
import subprocess
import sys

sys.path.append('doxy2swig')
from doxy2swig import Doxy2SWIG


member_methods = (
    ('gp_abilities_list_', '_CameraAbilitiesList', 'CameraAbilitiesList'),
    ('gp_camera_',         '_Camera',              'Camera'),
    ('gp_context_',        '_GPContext',           'Context'),
    ('gp_file_',           '_CameraFile',          'CameraFile'),
    ('gp_list_',           '_CameraList',          'CameraList'),
    ('gp_port_info_list_', '_GPPortInfoList',      'PortInfoList'),
    ('gp_port_info_',      '_GPPortInfo',          'PortInfo'),
    ('gp_widget_',         '_CameraWidget',        'CameraWidget'),
    )

def add_member_doc(symbol, value):
    for key, c_type, py_type in member_methods:
        if symbol.startswith(key):
            method = symbol.replace(key, '')
            if method == 'new':
                return ('%feature("docstring") {} "{}\n\n' +
                        'See also gphoto2.{}"\n\n').format(
                            symbol, value, py_type)
            return ('%feature("docstring") {} "{}\n\n' +
                    'See also gphoto2.{}.{}"\n\n' +
                    '%feature("docstring") {}::{} "{}\n\n' +
                    'See also gphoto2.{}"\n\n').format(
                        symbol, value, py_type, method,
                        c_type, method, value, symbol)
    return '%feature("docstring") {} "{}"\n\n'.format(symbol, value)

def main(argv=None):
    # get gphoto2 source to be processed
    if len(sys.argv) != 2:
        print('Usage: %s source_dir' % sys.argv[0])
        return 1
    src_dir = sys.argv[1]
    root_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    os.chdir(src_dir)
    print('doxygen', src_dir)
    subprocess.check_output(
        ['doxygen', os.path.join(root_dir, 'developer', 'Doxyfile')])
    os.chdir(root_dir)
    index_file = os.path.join(src_dir, 'doc', 'xml', 'index.xml')
    print('Doxy2SWIG ' + index_file)
    p = Doxy2SWIG(index_file,
                  with_function_signature = False,
                  with_type_info = False,
                  with_constructor_list = False,
                  with_attribute_list = False,
                  with_overloaded_functions = False,
                  textwidth = 72,
                  quiet = True)
    p.generate()
    text = ''.join(p.pieces)
    with open(os.path.join('src', 'gphoto2', 'common', 'doc.i'), 'w') as of:
        for match in re.finditer('%feature\("docstring"\) (\w+) \"(.+?)\";',
                                 text, re.DOTALL):
            symbol = match.group(1)
            value = match.group(2).strip()
            if not value:
                continue
            of.write(add_member_doc(symbol, value))
    return 0

if __name__ == "__main__":
    sys.exit(main())



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

from collections import defaultdict
import os
import sys
import unittest

import gphoto2 as gp

path = os.path.dirname(os.path.dirname(__file__))
if path not in sys.path:
    sys.path.insert(0, path)
from tests.vcamera import has_vcam, use_vcam


@unittest.skipUnless(has_vcam, 'no virtual camera')
class TestContext(unittest.TestCase):
    def setUp(self):
        use_vcam(True)

    def cb_idle(self, context, data):
        self.callback_count['cb_idle'] += 1
        self.assertEqual(data, 'A')

    def cb_error(self, context, text, data):
        self.callback_count['cb_error'] += 1
        self.assertEqual(data, 'B')

    def cb_status(self, context, text, data):
        self.callback_count['cb_status'] += 1
        self.assertEqual(data, 'C')

    def cb_message(self, context, text, data):
        self.callback_count['cb_message'] += 1
        self.assertEqual(data, 'D')

    def cb_question(self, context, text, data):
        self.callback_count['cb_question'] += 1
        self.assertEqual(data, 'E')
        return gp.GP_CONTEXT_FEEDBACK_OK

    def cb_cancel(self, context, data):
        self.callback_count['cb_cancel'] += 1
        self.assertEqual(data, 'F')
        return gp.GP_CONTEXT_FEEDBACK_OK

    def cb_progress_start(self, context, target, text, data):
        self.callback_count['cb_progress_start'] += 1
        self.assertEqual(data, 'G')
        return 123

    def cb_progress_update(self, context, id_, current, data):
        self.callback_count['cb_progress_update'] += 1
        self.assertEqual(data, 'G')
        self.assertEqual(id_, 123)

    def cb_progress_stop(self, context, id_, data):
        self.callback_count['cb_progress_stop'] += 1
        self.assertEqual(id_, 123)

    def test_oo_style(self):
        self.callback_count = defaultdict(int)
        context = gp.Context()
        # set callbacks
        callbacks = []
        callbacks.append(context.set_idle_func(self.cb_idle, 'A'))
        callbacks.append(context.set_error_func(self.cb_error, 'B'))
        callbacks.append(context.set_status_func(self.cb_status, 'C'))
        callbacks.append(context.set_message_func(self.cb_message, 'D'))
        callbacks.append(context.set_question_func(self.cb_question, 'E'))
        callbacks.append(context.set_cancel_func(self.cb_cancel, 'F'))
        callbacks.append(context.set_progress_funcs(
            self.cb_progress_start, self.cb_progress_update,
            self.cb_progress_stop, 'G'))
        for cb in callbacks:
            self.assertIsInstance(cb, gp.CallbackDetails)
        # create a virtual camera
        camera = gp.Camera()
        camera.init(context)
        # call some camera functions which may invoke some callbacks
        text = camera.get_summary(context)
        config = camera.get_config(context)
        path = camera.capture(gp.GP_CAPTURE_IMAGE, context)
        info = camera.file_get_info(path.folder, path.name, context)
        # all done
        camera.exit(context)
        del callbacks
        # check result
        self.assertEqual(self.callback_count['cb_progress_start'], 1)
        self.assertEqual(self.callback_count['cb_progress_stop'], 1)
        self.assertEqual(self.callback_count['cb_progress_update'], 20)
        self.assertEqual(self.callback_count['cb_cancel'], 30)

    def test_c_style(self):
        self.callback_count = defaultdict(int)
        context = gp.gp_context_new()
        # set callbacks
        callbacks = []
        callbacks.append(
            gp.gp_context_set_idle_func(context, self.cb_idle, 'A'))
        callbacks.append(
            gp.gp_context_set_error_func(context, self.cb_error, 'B'))
        callbacks.append(
            gp.gp_context_set_status_func(context, self.cb_status, 'C'))
        callbacks.append(
            gp.gp_context_set_message_func(context, self.cb_message, 'D'))
        callbacks.append(
            gp.gp_context_set_question_func(context, self.cb_question, 'E'))
        callbacks.append(
            gp.gp_context_set_cancel_func(context, self.cb_cancel, 'F'))
        callbacks.append(
            gp.gp_context_set_progress_funcs(
                context, self.cb_progress_start, self.cb_progress_update,
                self.cb_progress_stop, 'G'))
        for cb in callbacks:
            self.assertIsInstance(cb, gp.CallbackDetails)
        # create a virtual camera
        OK, camera = gp.gp_camera_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_camera_init(camera, context), gp.GP_OK)
        # call some camera functions which may invoke some callbacks
        OK, text = gp.gp_camera_get_summary(camera, context)
        self.assertEqual(OK, gp.GP_OK)
        OK, config = gp.gp_camera_get_config(camera, context)
        self.assertEqual(OK, gp.GP_OK)
        OK, path = gp.gp_camera_capture(camera, gp.GP_CAPTURE_IMAGE, context)
        self.assertEqual(OK, gp.GP_OK)
        OK, info = gp.gp_camera_file_get_info(
            camera, path.folder, path.name, context)
        self.assertEqual(OK, gp.GP_OK)
        # all done
        self.assertEqual(gp.gp_camera_exit(camera, context), gp.GP_OK)
        del callbacks
        # check result
        self.assertEqual(self.callback_count['cb_progress_start'], 1)
        self.assertEqual(self.callback_count['cb_progress_stop'], 1)
        self.assertEqual(self.callback_count['cb_progress_update'], 20)
        self.assertEqual(self.callback_count['cb_cancel'], 30)


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import unittest

import gphoto2 as gp


class TestList(unittest.TestCase):
    def test_oo_style(self):
        test_list = gp.CameraList()
        self.assertIsInstance(test_list, gp.CameraList)
        self.assertEqual(len(test_list), 0)
        test_list.append('B', '2')
        test_list.append('A', '1')
        test_list.append('D', None)
        self.assertEqual(len(test_list), 3)
        self.assertEqual(test_list.get_name(0), 'B')
        self.assertEqual(test_list.get_value(0), '2')
        self.assertEqual(test_list.get_name(1), 'A')
        self.assertEqual(test_list.get_value(1), '1')
        self.assertEqual(test_list.get_name(2), 'D')
        self.assertIsNone(test_list.get_value(2))
        self.assertEqual(test_list.find_by_name('B'), 0)
        self.assertEqual(test_list.find_by_name('A'), 1)
        self.assertEqual(test_list.find_by_name('D'), 2)
        with self.assertRaises(gp.GPhoto2Error) as cm:
            test_list.find_by_name('Z')
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR)
        keys = test_list.keys()
        self.assertEqual(len(keys), 3)
        self.assertEqual(keys[1], 'A')
        with self.assertRaises(IndexError):
            keys[3]
        self.assertEqual(tuple(keys), ('B', 'A', 'D'))
        self.assertEqual(tuple(test_list.values()), ('2', '1', None))
        self.assertEqual(tuple(test_list.items()),
                         (('B', '2'), ('A', '1'), ('D', None)))
        with self.assertRaises(IndexError):
            test_list[-4]
        self.assertEqual(test_list[-3], ('B', '2'))
        self.assertEqual(test_list[-2], ('A', '1'))
        self.assertEqual(test_list[-1], ('D', None))
        self.assertEqual(test_list[0], ('B', '2'))
        self.assertEqual(test_list[1], ('A', '1'))
        self.assertEqual(test_list[2], ('D', None))
        with self.assertRaises(IndexError):
            test_list[3]
        self.assertEqual(test_list['B'], '2')
        self.assertEqual(test_list['A'], '1')
        with self.assertRaises(KeyError):
            test_list['Z']
        it = iter(test_list)
        self.assertEqual(next(it), test_list[0])
        self.assertEqual(next(it), test_list[1])
        self.assertEqual(next(it), test_list[2])
        with self.assertRaises(StopIteration):
            next(it)
        test_list.sort()
        self.assertEqual(test_list.get_name(0), 'A')
        self.assertEqual(test_list.get_value(0), '1')
        test_list.set_name(0, 'C')
        test_list.set_value(0, '3')
        self.assertEqual(test_list.get_name(0), 'C')
        self.assertEqual(test_list.get_value(0), '3')
        test_list.reset()
        self.assertEqual(len(test_list), 0)

    def test_c_style(self):
        OK, test_list = gp.gp_list_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(test_list, gp.CameraList)
        self.assertEqual(gp.gp_list_count(test_list), 0)
        self.assertEqual(gp.gp_list_append(test_list, 'B', '2'), gp.GP_OK)
        self.assertEqual(gp.gp_list_append(test_list, 'A', '1'), gp.GP_OK)
        self.assertEqual(gp.gp_list_append(test_list, 'D', None), gp.GP_OK)
        self.assertEqual(gp.gp_list_count(test_list), 3)
        self.assertEqual(gp.gp_list_get_name(test_list, 0), [gp.GP_OK, 'B'])
        self.assertEqual(gp.gp_list_get_value(test_list, 0), [gp.GP_OK, '2'])
        self.assertEqual(gp.gp_list_get_name(test_list, 1), [gp.GP_OK, 'A'])
        self.assertEqual(gp.gp_list_get_value(test_list, 1), [gp.GP_OK, '1'])
        self.assertEqual(gp.gp_list_get_name(test_list, 2), [gp.GP_OK, 'D'])
        self.assertEqual(gp.gp_list_get_value(test_list, 2), [gp.GP_OK, None])
        self.assertEqual(gp.gp_list_find_by_name(test_list, 'B'), [gp.GP_OK, 0])
        self.assertEqual(gp.gp_list_find_by_name(test_list, 'A'), [gp.GP_OK, 1])
        self.assertEqual(gp.gp_list_find_by_name(test_list, 'D'), [gp.GP_OK, 2])
        self.assertEqual(
            gp.gp_list_find_by_name(test_list, 'Z'), [gp.GP_ERROR, 0])
        self.assertEqual(gp.gp_list_sort(test_list), gp.GP_OK)
        self.assertEqual(gp.gp_list_get_name(test_list, 0), [gp.GP_OK, 'A'])
        self.assertEqual(gp.gp_list_get_value(test_list, 0), [gp.GP_OK, '1'])
        self.assertEqual(gp.gp_list_set_name(test_list, 0, 'C'), gp.GP_OK)
        self.assertEqual(gp.gp_list_set_value(test_list, 0, '3'), gp.GP_OK)
        self.assertEqual(gp.gp_list_get_name(test_list, 0), [gp.GP_OK, 'C'])
        self.assertEqual(gp.gp_list_get_value(test_list, 0), [gp.GP_OK, '3'])
        self.assertEqual(gp.gp_list_reset(test_list), gp.GP_OK)
        self.assertEqual(gp.gp_list_count(test_list), 0)


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import locale
import os
import platform
import unittest

import gphoto2 as gp


class TestResult(unittest.TestCase):
    def test_exceptions(self):
        for error in (gp.GP_ERROR_MODEL_NOT_FOUND, gp.GP_ERROR_NO_MEMORY):
            with self.assertRaises(gp.GPhoto2Error) as cm:
                gp.check_result(error)
            ex = cm.exception
            self.assertEqual(ex.code, error)
            self.assertEqual(ex.string, gp.gp_result_as_string(error))

    def set_lang(self, lang):
        # setting environ works on Ubuntu, setlocale works on openSUSE
        # using both seems to be harmless
        os.environ['LC_ALL'] = lang
        os.environ['LANG'] = lang
        os.environ['LANGUAGE'] = lang
        # try default locale first
        try:
            if locale.setlocale(locale.LC_MESSAGES, '').startswith(lang):
                return
        except locale.Error:
            pass
        # try all known locales
        for k in locale.locale_alias:
            if k.startswith(lang):
                try:
                    locale.setlocale(
                        locale.LC_MESSAGES, locale.locale_alias[k])
                    return
                except locale.Error:
                    pass
        self.skipTest("failed to set language '{}'".format(lang))

    @unittest.skipIf(platform.system() == 'Darwin',
                     'locale setting fails on MacOS')
    def test_locales(self):
        self.set_lang('de')
        self.assertEqual(gp.gp_result_as_string(gp.GP_ERROR_MODEL_NOT_FOUND),
                         'Unbekanntes Modell')
        self.assertEqual(gp.gp_port_result_as_string(gp.GP_ERROR_NO_MEMORY),
                         'Speicher voll')
        self.set_lang('en')
        self.assertEqual(gp.gp_result_as_string(gp.GP_ERROR_MODEL_NOT_FOUND),
                         'Unknown model')
        self.assertEqual(gp.gp_port_result_as_string(gp.GP_ERROR_NO_MEMORY),
                         'Out of memory')


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import sys
import time
import unittest

import gphoto2 as gp

path = os.path.dirname(os.path.dirname(__file__))
if path not in sys.path:
    sys.path.insert(0, path)
from tests.vcamera import has_vcam, use_vcam


@unittest.skipUnless(has_vcam, 'no virtual camera')
class TestWidget(unittest.TestCase):
    def setUp(self):
        use_vcam(True)
        self.camera = gp.Camera()
        self.camera.init()

    def tearDown(self):
        self.camera.exit()

    def test_oo_style(self):
        # test almost every gphoto2.CameraWidget method
        # untested: get_range() & set_range() as no GP_WIDGET_RANGE widget
        config = self.camera.get_config()
        # test top level widget
        self.assertEqual(config.get_label(), 'Camera and Driver Configuration')
        self.assertEqual(config.get_name(), 'main')
        self.assertEqual(config.get_type(), gp.GP_WIDGET_WINDOW)
        self.assertIsInstance(config.get_id(), int)
        self.assertEqual(config.get_info(), '')
        self.assertEqual(config.count_children(), 6)
        # test section widgets
        actions = config.get_child_by_name('actions')
        self.assertEqual(actions.get_parent(), config)
        self.assertEqual(actions, config.get_child(0))
        actions_id = actions.get_id()
        self.assertEqual(actions, config.get_child_by_id(actions_id))
        self.assertEqual(actions.get_type(), gp.GP_WIDGET_SECTION)
        settings = config.get_child_by_name('settings')
        self.assertEqual(settings.get_type(), gp.GP_WIDGET_SECTION)
        self.assertEqual(settings.get_readonly(), 0)
        status = config.get_child_by_name('status')
        self.assertEqual(status.get_type(), gp.GP_WIDGET_SECTION)
        self.assertEqual(status.get_readonly(), 0)
        # test text widget
        widget = actions.get_child_by_label('Set Nikon Control Mode')
        self.assertEqual(widget.get_type(), gp.GP_WIDGET_TEXT)
        self.assertEqual(widget.get_value(), '0')
        self.assertEqual(widget.changed(), 0)
        widget.set_value('1')
        self.assertEqual(widget.get_value(), '1')
        self.assertEqual(widget.changed(), 1)
        self.assertEqual(widget.get_root(), config)
        # test toggle widget
        widget = actions.get_child_by_name('bulb')
        self.assertEqual(widget.get_type(), gp.GP_WIDGET_TOGGLE)
        self.assertEqual(widget.get_value(), 2)
        self.assertEqual(widget.changed(), 0)
        widget.set_value(0)
        self.assertEqual(widget.get_value(), 0)
        self.assertEqual(widget.changed(), 1)
        # test radio widget
        widget = settings.get_child_by_name('capturetarget')
        self.assertEqual(widget.get_type(), gp.GP_WIDGET_RADIO)
        self.assertEqual(widget.count_choices(), 2)
        choices = list(widget.get_choices())
        self.assertEqual(widget.get_choice(0), choices[0])
        self.assertEqual(widget.get_choice(1), choices[1])
        self.assertEqual(widget.get_value(), choices[0])
        self.assertEqual(widget.changed(), 0)
        widget.set_value(choices[1])
        self.assertEqual(widget.get_value(), choices[1])
        widget.set_value(choices[0])
        self.assertEqual(widget.get_value(), choices[0])
        self.assertEqual(widget.changed(), 1)
        widget.set_changed(0)
        self.assertEqual(widget.changed(), 0)
        # test date widget
        widget = settings.get_child_by_name('datetime')
        self.assertEqual(widget.get_type(), gp.GP_WIDGET_DATE)
        now = time.time()
        if time.localtime(now).tm_isdst == 1:
            # this needs testing on other time zones
            now -= 3600
        self.assertEqual(widget.get_value(), int(now))
        # test read-only widget
        widget = status.get_child_by_name('batterylevel')
        self.assertEqual(widget.get_type(), gp.GP_WIDGET_TEXT)
        self.assertEqual(widget.get_readonly(), 1)
        # store changed config
        self.camera.set_config(config)
        del config
        # test single config
        widget = self.camera.get_single_config('thumbsize')
        self.assertEqual(widget.get_type(), gp.GP_WIDGET_RADIO)
        self.assertEqual(widget.count_choices(), 2)
        choices = list(widget.get_choices())
        self.assertEqual(widget.get_choice(0), choices[0])
        self.assertEqual(widget.get_choice(1), choices[1])
        widget.set_value(choices[1])
        self.assertEqual(widget.get_value(), choices[1])
        self.assertEqual(widget.changed(), 1)
        self.camera.set_single_config('thumbsize', widget)
        del widget

    def test_c_style(self):
        # test almost every gphoto2.CameraWidget method
        # untested: get_range() & set_range() as no GP_WIDGET_RANGE widget
        OK, config = gp.gp_camera_get_config(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        # test top level widget
        self.assertEqual(gp.gp_widget_get_label(config),
                         [gp.GP_OK, 'Camera and Driver Configuration'])
        self.assertEqual(gp.gp_widget_get_name(config), [gp.GP_OK, 'main'])
        self.assertEqual(gp.gp_widget_get_type(config),
                         [gp.GP_OK, gp.GP_WIDGET_WINDOW])
        OK, config_id = gp.gp_widget_get_id(config)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(config_id, int)
        self.assertEqual(gp.gp_widget_get_info(config), [gp.GP_OK, ''])
        self.assertEqual(gp.gp_widget_count_children(config), 6)
        # test section widgets
        OK, actions = gp.gp_widget_get_child_by_name(config, 'actions')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_parent(actions), [gp.GP_OK, config])
        self.assertEqual(gp.gp_widget_get_child(config, 0), [gp.GP_OK, actions])
        OK, actions_id = gp.gp_widget_get_id(actions)
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_child_by_id(config, actions_id),
                         [gp.GP_OK, actions])
        self.assertEqual(gp.gp_widget_get_type(actions),
                         [gp.GP_OK, gp.GP_WIDGET_SECTION])
        OK, settings = gp.gp_widget_get_child_by_name(config, 'settings')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_type(settings),
                         [gp.GP_OK, gp.GP_WIDGET_SECTION])
        self.assertEqual(gp.gp_widget_get_readonly(settings), [gp.GP_OK, 0])
        OK, status = gp.gp_widget_get_child_by_name(config, 'status')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_type(status),
                         [gp.GP_OK, gp.GP_WIDGET_SECTION])
        self.assertEqual(gp.gp_widget_get_readonly(status), [gp.GP_OK, 0])
        # test text widget
        OK, widget = gp.gp_widget_get_child_by_label(
            actions, 'Set Nikon Control Mode')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_type(widget),
                         [gp.GP_OK, gp.GP_WIDGET_TEXT])
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, '0'])
        self.assertEqual(gp.gp_widget_changed(widget), 0)
        self.assertEqual(gp.gp_widget_set_value(widget, '1'), gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, '1'])
        self.assertEqual(gp.gp_widget_changed(widget), 1)
        self.assertEqual(gp.gp_widget_get_root(widget), [gp.GP_OK, config])
        # test toggle widget
        OK, widget = gp.gp_widget_get_child_by_name(actions, 'bulb')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_type(widget),
                         [gp.GP_OK, gp.GP_WIDGET_TOGGLE])
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, 2])
        self.assertEqual(gp.gp_widget_changed(widget), 0)
        self.assertEqual(gp.gp_widget_set_value(widget, 0), gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, 0])
        self.assertEqual(gp.gp_widget_changed(widget), 1)
        # test radio widget
        OK, widget = gp.gp_widget_get_child_by_name(settings, 'capturetarget')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_type(widget),
                         [gp.GP_OK, gp.GP_WIDGET_RADIO])
        self.assertEqual(gp.gp_widget_count_choices(widget), 2)
        choices = []
        for idx in range(gp.gp_widget_count_choices(widget)):
            OK, choice = gp.gp_widget_get_choice(widget, idx)
            self.assertEqual(OK, gp.GP_OK)
            self.assertIsInstance(choice, str)
            choices.append(choice)
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, choices[0]])
        self.assertEqual(gp.gp_widget_changed(widget), 0)
        self.assertEqual(gp.gp_widget_set_value(widget, choices[1]), gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, choices[1]])
        self.assertEqual(gp.gp_widget_changed(widget), 1)
        self.assertEqual(gp.gp_widget_set_value(widget, choices[0]), gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, choices[0]])
        self.assertEqual(gp.gp_widget_changed(widget), 1)
        self.assertEqual(gp.gp_widget_set_changed(widget, 0), gp.GP_OK)
        self.assertEqual(gp.gp_widget_changed(widget), 0)
        # test date widget
        OK, widget = gp.gp_widget_get_child_by_name(settings, 'datetime')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_type(widget),
                         [gp.GP_OK, gp.GP_WIDGET_DATE])
        now = time.time()
        if time.localtime(now).tm_isdst == 1:
            # this needs testing on other time zones
            now -= 3600
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, int(now)])
        # test read-only widget
        OK, widget = gp.gp_widget_get_child_by_name(status, 'batterylevel')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_type(widget),
                         [gp.GP_OK, gp.GP_WIDGET_TEXT])
        self.assertEqual(gp.gp_widget_get_readonly(widget), [gp.GP_OK, 1])
        # store changed config
        self.assertEqual(gp.gp_camera_set_config(self.camera, config), gp.GP_OK)
        del config
        # test single config
        OK, widget = gp.gp_camera_get_single_config(self.camera, 'thumbsize')
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_type(widget),
                         [gp.GP_OK, gp.GP_WIDGET_RADIO])
        self.assertEqual(gp.gp_widget_count_choices(widget), 2)
        choices = []
        for idx in range(gp.gp_widget_count_choices(widget)):
            OK, choice = gp.gp_widget_get_choice(widget, idx)
            self.assertEqual(OK, gp.GP_OK)
            self.assertIsInstance(choice, str)
            choices.append(choice)
        self.assertEqual(gp.gp_widget_set_value(widget, choices[1]), gp.GP_OK)
        self.assertEqual(gp.gp_widget_get_value(widget), [gp.GP_OK, choices[1]])
        self.assertEqual(gp.gp_widget_changed(widget), 1)
        self.assertEqual(gp.gp_camera_set_single_config(
            self.camera, 'thumbsize', widget), gp.GP_OK)
        del widget


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import sys
import time
import unittest

import gphoto2 as gp

path = os.path.dirname(os.path.dirname(__file__))
if path not in sys.path:
    sys.path.insert(0, path)
from tests.vcamera import has_vcam, use_vcam


@unittest.skipUnless(has_vcam, 'no virtual camera')
class TestFilesys(unittest.TestCase):
    def setUp(self):
        use_vcam(True)
        self.camera = gp.Camera()
        self.camera.init()

    def tearDown(self):
        self.camera.exit()

    def test_camera_storage_information(self):
        storage_info = self.camera.get_storageinfo()
        self.assertEqual(len(storage_info), 1)
        storage_info = storage_info[0]
        self.assertIsInstance(storage_info, gp.CameraStorageInformation)
        self.assertEqual(storage_info.fields,
                         gp.GP_STORAGEINFO_FREESPACEIMAGES |
                         gp.GP_STORAGEINFO_FREESPACEKBYTES|
                         gp.GP_STORAGEINFO_MAXCAPACITY|
                         gp.GP_STORAGEINFO_FILESYSTEMTYPE |
                         gp.GP_STORAGEINFO_STORAGETYPE |
                         gp.GP_STORAGEINFO_ACCESS |
                         gp.GP_STORAGEINFO_DESCRIPTION |
                         gp.GP_STORAGEINFO_LABEL |
                         gp.GP_STORAGEINFO_BASE)
        self.assertEqual(storage_info.basedir, '/store_00010001')
        self.assertEqual(storage_info.label, 'GPVCS Label')
        self.assertEqual(storage_info.description, 'GPVC Storage')
        self.assertEqual(storage_info.type, gp.GP_STORAGEINFO_ST_FIXED_RAM)
        self.assertEqual(storage_info.fstype, gp.GP_STORAGEINFO_FST_DCF)
        self.assertEqual(storage_info.access,
                         gp.GP_STORAGEINFO_AC_READONLY_WITH_DELETE)
        self.assertEqual(storage_info.capacitykbytes, 1085584)
        self.assertEqual(storage_info.freekbytes, 542792)
        self.assertEqual(storage_info.freeimages, 150)

    def test_camera_file_info(self):
        file_name = 'copyright-free-image.jpg'
        test_file = os.path.join(
            os.path.dirname(__file__), 'vcamera', file_name)
        camera_file_info = self.camera.file_get_info(
            '/store_00010001', file_name)
        self.assertIsInstance(camera_file_info, gp.CameraFileInfo)
        # check preview info
        preview_info = camera_file_info.preview
        self.assertIsInstance(preview_info, gp.CameraFileInfoPreview)
        self.assertEqual(preview_info.fields, gp.GP_FILE_INFO_TYPE)
        # check audio info
        audio_info = camera_file_info.audio
        self.assertIsInstance(audio_info, gp.CameraFileInfoAudio)
        self.assertEqual(audio_info.fields, gp.GP_FILE_INFO_NONE)
        # check file info
        file_info = camera_file_info.file
        self.assertIsInstance(file_info, gp.CameraFileInfoFile)
        self.assertEqual(file_info.fields,
                         gp.GP_FILE_INFO_MTIME | gp.GP_FILE_INFO_PERMISSIONS |
                         gp.GP_FILE_INFO_SIZE | gp.GP_FILE_INFO_TYPE)
        self.assertEqual(file_info.size, os.path.getsize(test_file))
        self.assertEqual(file_info.type, 'image/jpeg')
        self.assertEqual(file_info.permissions,
                         gp.GP_FILE_PERM_READ | gp.GP_FILE_PERM_DELETE)
        mtime = os.path.getmtime(test_file)
        if time.localtime(mtime).tm_isdst == 1:
            # this needs testing on other time zones
            mtime -= 3600
        self.assertEqual(file_info.mtime, int(mtime))


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2014-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

from collections import defaultdict
from setuptools import setup, Extension
from setuptools import __version__ as setuptools_version
import os
import subprocess
import sys

packages = ['gphoto2', 'gphoto2.examples']
package_dir = {'gphoto2.examples': 'examples'}
package_data = {'gphoto2.examples': ['*']}
exclude_package_data = {'': ['*.c']}
extra_link_args = []

if 'GPHOTO2_ROOT' in os.environ:
    # using a local build of libgphoto2
    gphoto2_dir = os.environ['GPHOTO2_ROOT']
    gphoto2_dir = os.path.expanduser(gphoto2_dir)
    if not os.path.isabs(gphoto2_dir):
        raise RuntimeError('GPHOTO2_ROOT is not an absolute path')
    print('Using libgphoto2 from {}'.format(gphoto2_dir))
    for root, dirs, files in os.walk(gphoto2_dir):
        if 'libgphoto2.pc' in files:
            os.environ['PKG_CONFIG_PATH'] = root
            break
    else:
        raise RuntimeError('No package config file found')
    # get gphoto2 libs
    lib_dir = subprocess.check_output(
        ['pkg-config', '--variable=libdir', 'libgphoto2'],
        universal_newlines=True).strip()
    packages.append('gphoto2.libgphoto2')
    package_dir['gphoto2.libgphoto2'] = os.path.relpath(lib_dir)
    package_data['gphoto2.libgphoto2'] = []
    # choose the libx.so.n versions of all the .so files and links
    for name in os.listdir(lib_dir):
        if (name.startswith('libgphoto') and '.so.' in name
                and len(name.split('.')) == 3):
            package_data['gphoto2.libgphoto2'].append(name)
    # get cam libs
    packages.append('gphoto2.libgphoto2.camlibs')
    lib_dir = subprocess.check_output(
        ['pkg-config', '--variable=driverdir', 'libgphoto2'],
        universal_newlines=True).strip()
    package_dir['gphoto2.libgphoto2.camlibs'] = os.path.relpath(lib_dir)
    package_data['gphoto2.libgphoto2.camlibs'] = ['*.so']
    # get io libs
    packages.append('gphoto2.libgphoto2.iolibs')
    lib_dir = subprocess.check_output(
        ['pkg-config', '--variable=driverdir', 'libgphoto2_port'],
        universal_newlines=True).strip()
    package_dir['gphoto2.libgphoto2.iolibs'] = os.path.relpath(lib_dir)
    package_data['gphoto2.libgphoto2.iolibs'] = []
    for name in os.listdir(lib_dir):
        if name == 'vusb.so':
            packages.append('gphoto2.libgphoto2.vusb')
            package_dir['gphoto2.libgphoto2.vusb'] = os.path.relpath(lib_dir)
            package_data['gphoto2.libgphoto2.vusb'] = [name]
        elif name.endswith('.so'):
            package_data['gphoto2.libgphoto2.iolibs'].append(name)
    # get localisation files
    prefix = subprocess.check_output(
        ['pkg-config', '--variable=prefix', 'libgphoto2'],
        universal_newlines=True).strip()
    locale_dir = os.path.join(prefix, 'share', 'locale')
    if os.path.isdir(locale_dir):
        packages.append('gphoto2.libgphoto2.locale')
        package_dir['gphoto2.libgphoto2.locale'] = os.path.relpath(locale_dir)
        package_data['gphoto2.libgphoto2.locale'] = [
            '*/LC_MESSAGES/libgphoto2*.mo']
        for name in os.listdir(locale_dir):
            packages.append(
                'gphoto2.libgphoto2.locale.' + name + '.LC_MESSAGES')
    # module compile options
    extra_link_args = ['-Wl,-rpath,$ORIGIN/libgphoto2']
    if sys.platform == 'linux':
        extra_link_args += ['-Wl,--disable-new-dtags']

cmd = ['pkg-config', '--modversion', 'libgphoto2']
FNULL = open(os.devnull, 'w')
try:
    gphoto2_version_str = subprocess.check_output(
        cmd, stderr=FNULL, universal_newlines=True).strip()
except Exception:
    raise RuntimeError('ERROR: command "%s" failed' % ' '.join(cmd))
print('Using libgphoto2 v{}'.format(gphoto2_version_str))
# get libgphoto2 config
gphoto2_flags = defaultdict(list)
for flag in subprocess.check_output(
        ['pkg-config', '--cflags', '--libs', 'libgphoto2'],
        universal_newlines=True).split():
    gphoto2_flags[flag[:2]].append(flag)
gphoto2_include  = gphoto2_flags['-I']
for n in range(len(gphoto2_include)):
    if gphoto2_include[n].endswith('/gphoto2'):
        gphoto2_include[n] = gphoto2_include[n][:-len('/gphoto2')]
libraries = [x.replace('-l', '') for x in gphoto2_flags['-l']]
library_dirs = [x.replace('-L', '') for x in gphoto2_flags['-L']]
include_dirs = [x.replace('-I', '') for x in gphoto2_include]

# get list of available swigged versions
swigged_versions = []
for name in os.listdir('src'):
    if not name.startswith('swig-gp'):
        continue
    swigged_version = name.replace('swig-gp', '').split('_')
    swigged_versions.append(tuple(map(int, swigged_version)))
swigged_versions.sort()

# choose best match from swigged versions
gphoto2_version = tuple(map(int, gphoto2_version_str.split('.')))
if gphoto2_version < (2, 5, 10):
    raise RuntimeError('libgphoto2 version 2.5.10 or later is required')
while len(swigged_versions) > 1 and swigged_versions[0] < gphoto2_version:
    swigged_versions = swigged_versions[1:]
swigged_version = swigged_versions[0]

# create extension modules list
ext_modules = []
mod_src_dir = 'swig-gp' + '_'.join(map(str, swigged_version))
mod_src_dir = os.path.join('src', mod_src_dir)
package_dir['gphoto2'] = mod_src_dir

extra_compile_args = [
    '-O3', '-Wno-unused-variable', '-Wno-unused-but-set-variable',
    '-Wno-unused-label', '-Wno-strict-prototypes']
if 'PYTHON_GPHOTO2_STRICT' in os.environ:
    extra_compile_args.append('-Werror')
define_macros = [('GPHOTO2_VERSION',
                  '0x{:02x}{:02x}{:02x}{:02x}'.format(*gphoto2_version, 0, 0)),
                 ('SWIG_TYPE_TABLE', 'gphoto2')]
for file_name in os.listdir(mod_src_dir):
    if file_name[-7:] != '_wrap.c':
        continue
    ext_name = file_name[:-7]
    ext_modules.append(Extension(
        '_' + ext_name,
        sources = [os.path.join(mod_src_dir, file_name)],
        libraries = libraries,
        library_dirs = library_dirs,
        include_dirs = include_dirs,
        extra_compile_args = extra_compile_args,
        define_macros = define_macros,
        extra_link_args = extra_link_args,
        ))

setup_kwds = {
    'ext_package': 'gphoto2',
    'ext_modules': ext_modules,
    'packages': packages,
    'package_dir': package_dir,
    'package_data': package_data,
    'exclude_package_data': exclude_package_data,
    'include_package_data': False,
    }

if tuple(map(int, setuptools_version.split('.')[:2])) < (61, 0):
    # get metadata from pyproject.toml
    import toml
    metadata = toml.load('pyproject.toml')

    with open(metadata['project']['readme']) as ldf:
        long_description = ldf.read()
        # python-gphoto2 version
        version = long_description.split('\n')[0].split()[-1]

    setup_kwds.update(
        name = metadata['project']['name'],
        version = version,
        description = metadata['project']['description'],
        long_description = long_description,
        author = metadata['project']['authors'][0]['name'],
        author_email = metadata['project']['authors'][0]['email'],
        url = metadata['project']['urls']['homepage'],
        classifiers = metadata['project']['classifiers'],
        platforms = metadata['tool']['setuptools']['platforms'],
        license = metadata['project']['license']['text'],
        zip_safe = metadata['tool']['setuptools']['zip-safe'],
        )

setup(**setup_kwds)



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import sys
import time
import unittest

import gphoto2 as gp

path = os.path.dirname(os.path.dirname(__file__))
if path not in sys.path:
    sys.path.insert(0, path)
from tests.vcamera import has_vcam, use_vcam


class TestNoCamera(unittest.TestCase):
    def setUp(self):
        use_vcam(False)

    def test_oo_style(self):
        cameras = gp.Camera.autodetect()
        self.assertIsInstance(cameras, gp.CameraList)
        camera = gp.Camera()
        if len(cameras) > 0:
            # there is actually a camera connected
            camera.init()
            camera.exit()
        else:
            with self.assertRaises(gp.GPhoto2Error) as cm:
                camera.init()
            ex = cm.exception
            self.assertEqual(ex.code, gp.GP_ERROR_MODEL_NOT_FOUND)

    def test_c_style(self):
        OK, cameras = gp.gp_camera_autodetect()
        self.assertGreaterEqual(OK, gp.GP_OK)
        self.assertIsInstance(cameras, gp.CameraList)
        OK, camera = gp.gp_camera_new()
        self.assertEqual(OK, gp.GP_OK)
        OK = gp.gp_camera_init(camera)
        if len(cameras) > 0:
            # there is actually a camera connected
            self.assertEqual(OK, gp.GP_OK)
            self.assertEqual(gp.gp_camera_exit(camera), gp.GP_OK)
        else:
            self.assertEqual(OK, gp.GP_ERROR_MODEL_NOT_FOUND)


@unittest.skipUnless(has_vcam, 'no virtual camera')
class TestAutoDetect(unittest.TestCase):
    def setUp(self):
        use_vcam(True)

    def test_oo_style(self):
        cameras = gp.Camera.autodetect()
        self.assertIsInstance(cameras, gp.CameraList)
        self.assertEqual(len(cameras), 1)
        name, value = cameras[0]
        self.assertEqual(name, 'Nikon DSC D750')

    def test_c_style(self):
        OK, cameras = gp.gp_camera_autodetect()
        self.assertGreaterEqual(OK, gp.GP_OK)
        self.assertIsInstance(cameras, gp.CameraList)
        self.assertEqual(gp.gp_list_count(cameras), 1)
        OK, name = gp.gp_list_get_name(cameras, 0)
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(name, 'Nikon DSC D750')


@unittest.skipUnless(has_vcam, 'no virtual camera')
class TestVirtualCamera(unittest.TestCase):
    def setUp(self):
        use_vcam(True)
        self.camera = gp.Camera()
        self.camera.init()

    def tearDown(self):
        self.camera.exit()

    def test_oo_style(self):
        def list_files(path='/'):
            result = []
            # get files
            for name in self.camera.folder_list_files(path).keys():
                result.append(os.path.join(path, name))
            # recurse over subfolders
            for name in self.camera.folder_list_folders(path).keys():
                result.extend(list_files(os.path.join(path, name)))
            return result

        # get_about
        text = str(self.camera.get_about())
        self.assertTrue(text.startswith('PTP2 driver'))
        self.assertTrue(text.endswith('Enjoy!'))
        # get_summary
        text = str(self.camera.get_summary())
        self.assertTrue(text.startswith('Manufacturer'))
        # get manual
        with self.assertRaises(gp.GPhoto2Error) as cm:
            manual = self.camera.get_manual()
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR_NOT_SUPPORTED)
        # get storage info
        storage_info = self.camera.get_storageinfo()
        self.assertEqual(len(storage_info), 1)
        storage_info = storage_info[0]
        self.assertIsInstance(storage_info, gp.CameraStorageInformation)
        # get_config
        widget = self.camera.get_config()
        self.assertIsInstance(widget, gp.CameraWidget)
        self.camera.set_config(widget)
        # get single config
        widget = self.camera.get_single_config('thumbsize')
        self.assertIsInstance(widget, gp.CameraWidget)
        # list config
        config_list = self.camera.list_config()
        self.assertEqual(len(config_list), 68)
        # list_files
        files = list_files()
        self.assertEqual(files[0], '/store_00010001/copyright-free-image.jpg')
        # file_get
        file = self.camera.file_get(
            '/store_00010001', 'copyright-free-image.jpg',
            gp.GP_FILE_TYPE_NORMAL)
        self.assertIsInstance(file, gp.CameraFile)
        # file put
        with self.assertRaises(gp.GPhoto2Error) as cm:
            self.camera.folder_put_file('/store_00010001', 'uploaded.jpg',
                                        gp.GP_FILE_TYPE_NORMAL, file)
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR_NOT_SUPPORTED)
        # file_read
        buffer = bytearray(10)
        with self.assertRaises(gp.GPhoto2Error) as cm:
            data = self.camera.file_read(
                '/store_00010001', 'copyright-free-image.jpg',
                gp.GP_FILE_TYPE_NORMAL, 0, buffer)
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR_NOT_SUPPORTED)
        # capture
        path = self.camera.capture(gp.GP_CAPTURE_IMAGE)
        self.assertRegex(path.name, 'GPH_\d{4}.JPG')
        self.assertRegex(path.folder, '/store_00010001/DCIM/\d{3}GPHOT')
        info = self.camera.file_get_info(path.folder, path.name)
        self.assertEqual(info.file.size, 7082)
        self.assertEqual(info.file.type, 'image/jpeg')
        # width and height are zero as test image has no exif info
        self.assertEqual(info.file.width, 0)
        self.assertEqual(info.file.height, 0)
        # file set info
        new_info = gp.CameraFileInfo()
        new_info.preview.fields = gp.GP_FILE_INFO_NONE
        new_info.audio.fields = gp.GP_FILE_INFO_NONE
        new_info.file.fields = gp.GP_FILE_INFO_MTIME
        new_info.file.mtime = int(time.time())
        self.camera.file_set_info(path.folder, path.name, new_info)
        # capture preview
        with self.assertRaises(gp.GPhoto2Error) as cm:
            preview_file = self.camera.capture_preview()
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR_NOT_SUPPORTED)
        # trigger capture
        self.camera.trigger_capture()
        # wait for event
        while True:
            event_type, event_data = self.camera.wait_for_event(100)
            if event_type in (gp.GP_EVENT_TIMEOUT,
                              gp.GP_EVENT_CAPTURE_COMPLETE):
                break
            if event_type in (gp.GP_EVENT_FILE_ADDED,
                              gp.GP_EVENT_FOLDER_ADDED):
                self.assertIsInstance(event_data, gp.CameraFilePath)
        # delete file(s)
        self.camera.file_delete(path.folder, path.name)
        self.camera.folder_delete_all(path.folder)
        # create folder
        folder, name = '/store_00010001', 'new folder'
        with self.assertRaises(gp.GPhoto2Error) as cm:
            self.camera.folder_make_dir(folder, name)
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR_NOT_SUPPORTED)
        # delete folder
        with self.assertRaises(gp.GPhoto2Error) as cm:
            self.camera.folder_remove_dir(folder, name)
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR_DIRECTORY_NOT_FOUND)
        # port info
        info = self.camera.get_port_info()
        self.assertIsInstance(info, gp.PortInfo)
        with self.assertRaises(gp.GPhoto2Error) as cm:
            self.camera.set_port_info(info)
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR_LIBRARY)
        # port speed
        speed = self.camera.get_port_speed()
        self.assertEqual(speed, 0)
        with self.assertRaises(gp.GPhoto2Error) as cm:
            self.camera.set_port_speed(0)
        ex = cm.exception
        self.assertEqual(ex.code, gp.GP_ERROR_BAD_PARAMETERS)
        # abilities list
        abilities = self.camera.get_abilities()
        self.assertIsInstance(abilities, gp.CameraAbilities)
        self.camera.set_abilities(abilities)

    def test_c_style(self):
        def list_files(path='/'):
            result = []
            # get files
            OK, files = gp.gp_camera_folder_list_files(self.camera, path)
            self.assertEqual(OK, gp.GP_OK)
            for idx in range(gp.gp_list_count(files)):
                OK, name = gp.gp_list_get_name(files, idx)
                self.assertEqual(OK, gp.GP_OK)
                result.append(os.path.join(path, name))
            # recurse over subfolders
            OK, folders = gp.gp_camera_folder_list_folders(self.camera, path)
            self.assertEqual(OK, gp.GP_OK)
            for idx in range(gp.gp_list_count(folders)):
                OK, name = gp.gp_list_get_name(folders, idx)
                self.assertEqual(OK, gp.GP_OK)
                result.extend(list_files(os.path.join(path, name)))
            return result

        # get_about
        OK, text = gp.gp_camera_get_about(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        self.assertTrue(text.text.startswith('PTP2 driver'))
        self.assertTrue(text.text.endswith('Enjoy!'))
        # get_summary
        OK, text = gp.gp_camera_get_summary(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        self.assertTrue(text.text.startswith('Manufacturer'))
        # get manual
        OK, manual = gp.gp_camera_get_manual(self.camera)
        self.assertEqual(OK, gp.GP_ERROR_NOT_SUPPORTED)
        # get storage info
        OK, storage_info = gp.gp_camera_get_storageinfo(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(len(storage_info), 1)
        storage_info = storage_info[0]
        self.assertIsInstance(storage_info, gp.CameraStorageInformation)
        # get_config
        OK, widget = gp.gp_camera_get_config(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(widget, gp.CameraWidget)
        self.assertEqual(gp.gp_camera_set_config(self.camera, widget), gp.GP_OK)
        # get single config
        OK, widget = gp.gp_camera_get_single_config(self.camera, 'thumbsize')
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(widget, gp.CameraWidget)
        # list config
        OK, config_list = gp.gp_camera_list_config(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(gp.gp_list_count(config_list), 68)
        # list_files
        files = list_files()
        self.assertEqual(files[0], '/store_00010001/copyright-free-image.jpg')
        # file_get
        OK, file = gp.gp_camera_file_get(
            self.camera, '/store_00010001', 'copyright-free-image.jpg',
            gp.GP_FILE_TYPE_NORMAL)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(file, gp.CameraFile)
        # file put
        self.assertEqual(
            gp.gp_camera_folder_put_file(
                self.camera, '/store_00010001', 'uploaded.jpg',
                gp.GP_FILE_TYPE_NORMAL, file),
            gp.GP_ERROR_NOT_SUPPORTED)
        # file_read
        buffer = bytearray(10)
        OK, data = gp.gp_camera_file_read(
            self.camera, '/store_00010001', 'copyright-free-image.jpg',
            gp.GP_FILE_TYPE_NORMAL, 0, buffer)
        self.assertEqual(OK, gp.GP_ERROR_NOT_SUPPORTED)
        # capture
        OK, path = gp.gp_camera_capture(self.camera, gp.GP_CAPTURE_IMAGE)
        self.assertEqual(OK, gp.GP_OK)
        self.assertRegex(path.name, 'GPH_\d{4}.JPG')
        self.assertRegex(path.folder, '/store_00010001/DCIM/\d{3}GPHOT')
        OK, info = gp.gp_camera_file_get_info(
            self.camera, path.folder, path.name)
        self.assertEqual(OK, gp.GP_OK)
        self.assertEqual(info.file.size, 7082)
        self.assertEqual(info.file.type, 'image/jpeg')
        # width and height are zero as test image has no exif info
        self.assertEqual(info.file.width, 0)
        self.assertEqual(info.file.height, 0)
        # file set info
        new_info = gp.CameraFileInfo()
        new_info.preview.fields = gp.GP_FILE_INFO_NONE
        new_info.audio.fields = gp.GP_FILE_INFO_NONE
        new_info.file.fields = gp.GP_FILE_INFO_MTIME
        new_info.file.mtime = int(time.time())
        OK = gp.gp_camera_file_set_info(
            self.camera, path.folder, path.name, new_info)
        self.assertEqual(OK, gp.GP_OK)
        # capture preview
        OK, preview_file = gp.gp_camera_capture_preview(self.camera)
        self.assertEqual(OK, gp.GP_ERROR_NOT_SUPPORTED)
        # trigger capture
        self.assertEqual(gp.gp_camera_trigger_capture(self.camera), gp.GP_OK)
        # wait for event
        while True:
            OK, event_type, event_data = gp.gp_camera_wait_for_event(
                self.camera, 100)
            self.assertEqual(OK, gp.GP_OK)
            if event_type in (gp.GP_EVENT_TIMEOUT,
                              gp.GP_EVENT_CAPTURE_COMPLETE):
                break
            if event_type in (gp.GP_EVENT_FILE_ADDED,
                              gp.GP_EVENT_FOLDER_ADDED):
                self.assertIsInstance(event_data, gp.CameraFilePath)
        # delete file(s)
        self.assertEqual(gp.gp_camera_file_delete(
            self.camera, path.folder, path.name), gp.GP_OK)
        self.assertEqual(gp.gp_camera_folder_delete_all(
            self.camera, path.folder), gp.GP_OK)
        # create folder
        folder, name = '/store_00010001', 'new folder'
        self.assertEqual(
            gp.gp_camera_folder_make_dir(self.camera, folder, name),
            gp.GP_ERROR_NOT_SUPPORTED)
        # delete folder
        self.assertEqual(
            gp.gp_camera_folder_remove_dir(self.camera, folder, name),
            gp.GP_ERROR_DIRECTORY_NOT_FOUND)
        # port info
        OK, info = gp.gp_camera_get_port_info(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(info, gp.PortInfo)
        self.assertEqual(
            gp.gp_camera_set_port_info(self.camera, info), gp.GP_ERROR_LIBRARY)
        # port speed
        speed = gp.gp_camera_get_port_speed(self.camera)
        self.assertEqual(speed, 0)
        self.assertEqual(gp.gp_camera_set_port_speed(self.camera, 0),
                         gp.GP_ERROR_BAD_PARAMETERS)
        # abilities list
        OK, abilities = gp.gp_camera_get_abilities(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(abilities, gp.CameraAbilities)
        self.assertEqual(
            gp.gp_camera_set_abilities(self.camera, abilities), gp.GP_OK)


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import logging
import os
import sys
import unittest

import gphoto2 as gp

path = os.path.dirname(os.path.dirname(__file__))
if path not in sys.path:
    sys.path.insert(0, path)
from tests.vcamera import has_vcam, use_vcam


@unittest.skipUnless(has_vcam, 'no virtual camera')
class TestPort(unittest.TestCase):
    def setUp(self):
        use_vcam(True)
        self.camera = gp.Camera()
        self.camera.init()
        self.camera.exit()

    def test_oo_style(self):
        info = self.camera.get_port_info()
        port = gp.GPPort()
        self.assertIsInstance(port, gp.GPPort)
        port.set_info(info)
        info_copy = port.get_info()
        self.assertIsInstance(info_copy, gp.PortInfo)
        port.open()
        port.reset()
        port.close()

    def test_c_style(self):
        OK, info = gp.gp_camera_get_port_info(self.camera)
        self.assertEqual(OK, gp.GP_OK)
        OK, port = gp.gp_port_new()
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(port, gp.GPPort)
        self.assertEqual(gp.gp_port_set_info(port, info), gp.GP_OK)
        OK, info_copy = gp.gp_port_get_info(port)
        self.assertEqual(OK, gp.GP_OK)
        self.assertIsInstance(info_copy, gp.PortInfo)
        self.assertEqual(gp.gp_port_open(port), gp.GP_OK)
        self.assertEqual(gp.gp_port_reset(port), gp.GP_OK)
        self.assertEqual(gp.gp_port_close(port), gp.GP_OK)


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2023  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import unittest

import gphoto2 as gp


class TestVersion(unittest.TestCase):
    def test_version(self):
        version = gp.gp_library_version(gp.GP_VERSION_SHORT)
        self.assertIsInstance(version, list)
        self.assertIsInstance(version[0], str)
        version = gp.gp_port_library_version(gp.GP_VERSION_SHORT)
        self.assertIsInstance(version, list)
        self.assertIsInstance(version[0], str)


if __name__ == "__main__":
    unittest.main()



# python-gphoto2 - Python interface to libgphoto2
# http://github.com/jim-easterbrook/python-gphoto2
# Copyright (C) 2021-23  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This file is part of python-gphoto2.
#
# python-gphoto2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# python-gphoto2 is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with python-gphoto2.  If not, see
# <https://www.gnu.org/licenses/>.

import os
import shutil
import subprocess
import sys


def main(argv=None):
    # system or local libexiv2?
    if len(sys.argv) > 2:
        print('Usage: %s [prefix]' % sys.argv[0])
        return 1
    if len(sys.argv) > 1:
        for root, dirs, files in os.walk(sys.argv[1]):
            if 'libgphoto2.pc' in files:
                os.environ['PKG_CONFIG_PATH'] = root
                break
        else:
            print('No package config file found')
            return 2
    # get python-gphoto2 version
    with open('README.rst') as rst:
        version = rst.readline().split()[-1]
    # get libgphoto2 version to be swigged
    cmd = ['pkg-config', '--modversion', 'libgphoto2']
    FNULL = open(os.devnull, 'w')
    try:
        gphoto2_version_str = subprocess.check_output(
            cmd, stderr=FNULL, universal_newlines=True).strip()
    except Exception:
        print('ERROR: command "{}" failed'.format(' '.join(cmd)))
        raise
    gphoto2_include = subprocess.check_output(
            ['pkg-config', '--cflags-only-I', 'libgphoto2'],
            universal_newlines=True).strip().split()
    for n in range(len(gphoto2_include)):
        if gphoto2_include[n].endswith('/gphoto2'):
            gphoto2_include[n] = gphoto2_include[n][:-len('/gphoto2')]
    # get list of modules (Python) and extensions (SWIG)
    file_names = os.listdir(os.path.join('src', 'gphoto2'))
    file_names.sort()
    file_names = [os.path.splitext(x) for x in file_names]
    ext_names = [x[0] for x in file_names if x[1] == '.i']
    py_names = [x[0] + x[1] for x in file_names if x[1] == '.py']
    # get SWIG version
    cmd = ['swig', '-version']
    try:
        swig_version = str(subprocess.Popen(
            cmd, stdout=subprocess.PIPE,
            universal_newlines=True).communicate()[0])
    except Exception:
        print('ERROR: command "%s" failed' % ' '.join(cmd))
        raise
    for line in swig_version.splitlines():
        if 'Version' in line:
            swig_version = tuple(map(int, line.split()[-1].split('.')))
            break
    # make options list
    swig_opts = ['-python', '-nodefaultctor', '-O',
                 '-Wextra', '-Werror', '-builtin', '-nofastunpack']
    if swig_version < (4, 1, 0):
        swig_opts.append('-py3')
    doc_file = os.path.join('src', 'gphoto2', 'common', 'doc.i')
    output_dir = os.path.join('src', 'swig')
    output_dir += '-gp' + gphoto2_version_str.replace('.', '_')
    os.makedirs(output_dir, exist_ok=True)
    version_opts = ['-outdir', output_dir]
    if os.path.isfile(doc_file):
        version_opts.append(
            '-DDOC_FILE=' + os.path.basename(doc_file))
    version_opts += gphoto2_include
    # do each swig module
    for ext_name in ext_names:
        cmd = ['swig'] + swig_opts + version_opts + ['-o']
        cmd += [os.path.join(output_dir, ext_name + '_wrap.c')]
        cmd += [os.path.join('src', 'gphoto2', ext_name + '.i')]
        print(' '.join(cmd))
        subprocess.check_output(cmd)
    # copy python modules
    for py_name in py_names:
        shutil.copy2(os.path.join('src', 'gphoto2', py_name), output_dir)
    # create init module
    init_file = os.path.join(output_dir, '__init__.py')
    with open(init_file, 'w') as im:
        im.write('__version__ = "{}"\n'.format(version))
        im.write('__version_tuple__ = tuple(({}))\n'.format(
            ', '.join(version.split('.'))))
        im.write('''

import os

_dir = os.path.dirname(__file__)
_camlibs = os.path.join(_dir, 'libgphoto2', 'camlibs')
if os.path.isdir(_camlibs):
    os.environ['CAMLIBS'] = _camlibs
if 'VCAMERADIR' in os.environ:
    _iolibs = os.path.join(_dir, 'libgphoto2', 'vusb')
else:
    _iolibs = os.path.join(_dir, 'libgphoto2', 'iolibs')
if os.path.isdir(_iolibs):
    os.environ['IOLIBS'] = _iolibs

class GPhoto2Error(Exception):
    """Exception raised by gphoto2 library errors

    Attributes:
        code   (int): the gphoto2 error code
        string (str): corresponding error message
    """
    def __init__(self, code):
        string = gp_result_as_string(code)
        Exception.__init__(self, '[%d] %s' % (code, string))
        self.code = code
        self.string = string

''')
        for name in ext_names:
            im.write('from gphoto2.{} import *\n'.format(name))
        im.write('''
_locale = os.path.join(_dir, 'libgphoto2', 'locale')
if os.path.isdir(_locale):
    gphoto2.abilities_list.gp_init_localedir(_locale)

__all__ = dir()
''')
    return 0


if __name__ == "__main__":
    sys.exit(main())
